'''
Basic realscanners for sagator, version 0.6.2

(c) 2003-2005 Jan ONDREJ (SAL) <ondrejj(at)salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
    
'''

from avlib import *
import re,os,time,socket,filetype

__all__=['regexp_scan','string_scan','smtp_comm',
         'const','max_file_size','exec_any',
         'file_type','file_magic']

def to_list(args,fx=None):
    ''' Convert dictionary to list if type is dict or return original. '''
    if type(args)==type({}):
      ret=[]
      for vname,vals in args.items():
        if fx!=None:
          ret.append([vname]+map(fx,vals))
        else:
          ret.append([vname]+vals)
      return ret
    elif fx!=None:
      ret=[]
      for arg in args:
        ret.append([arg[0]]+map(fx,arg[1:]))
      return ret
    else:
      return args

class regexp_scan(ascanner):
  '''
  Primitive regexp pattern scanner.
  
  There can be more patterns for one virus. All patterns in []
  must match to assign an stream as virus (AND opeator).
  There can also be more virnames in one dictionary.

  Usage: regexp_scan([['VirName','RegExp_Pattern1...',...],...],size=0,flags=0)

  Where: 'VirName' is a string, which identifies defined virus
         'RegExp_Pattern1...' is a regexp pattern
         size is a number, which defines, how many bytes may be checked.
           If it is 0 or not defined, whole stream is scanned.
           If it is -1, email header is scanned.
         flags is an integer, which defines regular expression flags.
           By default no flags are used.
    
  Example: regexp_scan([
             # Scan for a part of EICAR virus test file pattern
             ['EICAR','^X5O!P%@AP[4.*EICAR-STANDARD-ANTIVIRUS-TEST-FILE'],
             # Scan for a an EXE file pattern endoded as base64.
             ['UnknownEXE','^TVqQ']
           ])
  '''
  name='RegExpScanner()'
  def __init__(self,regexps,size=0,flags=0):
      self.size=size
      self.flags=re.M|flags
      # compile regexps
      self.regexps=to_list(regexps,self.reg_compile)
  def reg_compile(self,arg):
      return re.compile(arg,self.flags)
  def scanstream(self,stream):
      for regs in self.regexps:
        for reg in regs[1:]:
          if self.size>0:
            reg1=reg.search(stream[:self.size])
          elif self.size<0:
            reg1=reg.search(stream[:mail.bodypos])
          else:
            reg1=reg.search(stream)
          if reg1:
            if reg==regs[-1]:
              mail.addheader("X-Sagator-RegExp",regs[0])
              return iret(1.0,regs[0],['regexp: '+regs[0]+' FOUND!\n'])
            else:
              break
      return 0.0,'',[]

class string_scan(ascanner):
  '''
  Primitive string pattern scanner.
  
  There can be more patterns for one virus. All patterns
  must match to assign an stream as virus.
  There can also be more virnames in one dictionary.

  Usage: string_scan([['VirName','Pattern1...',...],...],size=0)

  Where: 'VirName' is a string, which identifies defined virus
         'Pattern...' is a string pattern
         size is a number, which defines, how many bytes may be checked.
           If it is 0 or not defined, whole stream is scanned.
           If it is -1, email header is scanned.
    
  Example: string_scan([
             # Scan for a part of EICAR virus test file pattern
             ['EICAR','X5O!P%@AP[4','EICAR-STANDARD-ANTIVIRUS-TEST-FILE'],
             # Scan for a an EXE file pattern endoded as base64.
             ['UnknownEXE','TVqQ']
           ])
  '''
  name='StringScanner()'
  def __init__(self,strings,size=0):
      self.strings=to_list(strings)
      self.size=size
  def scanstream(self,stream):
      for regs in self.strings:
        for reg in regs[1:]:
          if self.size>0:
            str1=stream[:self.size].find(reg)
          elif self.size<0:
            str1=stream[:mail.bodypos].find(reg)
          else:
            str1=stream.find(reg)
          if str1>=0:
            if reg==regs[-1]:
              mail.addheader("X-Sagator-StringScan",regs[0])
              return iret(1.0,regs[0],['string: '+regs[0]+' FOUND!\n'])
          else:
            break
      return 0.0,'',[]

class smtp_comm(ascanner):
  '''
  Primitive regexp pattern scanner for SMTP communication.
  
  For more information about this scanner see documentation for
  regexp_scan realscanner.
  
  This scanner also can be used to apply some scnanner to defined
  email adresses. For example:
      smtp_comm(['D','^RCPT TO:.*anydomain.dom']])
        & stream2mbox(libclam())
  In this example, libclam() scanner is used only if mail is
  adressed to an user on anydomain.dom.

  Usage: smtp_comm([['VirName','RegExp_Pattern...',...],...],flags=0)

  Where: 'VirName' is a string, which identifies defined virus
         'RegExp_Pattern...' is a regexp pattern
         flags is an integer, which defines regular expression flags.
           By default re.IGNORECASE is used.
  '''
  name='smtp_comm()'
  def __init__(self,regexps,flags=re.I):
      self.flags=flags
      # compile regexps
      self.regexps=to_list(regexps,self.reg_compile)
  def reg_compile(self,arg):
      return re.compile(arg,self.flags)
  def scanstream(self,stream):
      if globals.scan_only:
        return 0.0,'',[]
      for regs in self.regexps:
        for reg in regs[1:]:
          if reg.search(mail.comm):
            if reg==regs[-1]:
              mail.addheader("X-Sagator-SMTP-comm",regs[0])
              return iret(1.0,regs[0],['smtp_comm: '+regs[0]+' FOUND!\n'])
          else:
            break
      return 0.0,'',[]
  def scanfile(self,files,dir=''):
      return self.scanstream('')

#####################################################################
### primitive class

class const(ascanner):
  '''
  Realscanner to return a constant value (virus or clean).
  
  This scanner has no special functionality. It returns always
  a defined virus, or clean (if virus is not defined).
  This scanner has no error codes.
  
  Usage: const(level,VirName,return_string=[])
      or const()
  
  Where: level is an float which defines returned infection level
           If level is not defined, after scanning an error is raised.
         VirName is a returned virus.
         return_string is a array of strings returned
  
  Examples: const(0.0)         # Return clean
            const(1.0,'Virus') # Return virus name "Virus"
            const()            # Raise an error
  '''
  name='const()'
  ignore_name=True
  def __init__(self,level=None,vir='',ret=[]):
      self.level=level
      self.vir=vir
      self.ret=ret
      self.filename=''
  def scanstream(self,stream):
      if self.level==None:
        raise ScannerError,'Forced error by const() scanner'
      if self.filename:
        debug.echo(6,self.name,": Scanning file: ",self.filename)
      return self.level,self.vir,self.ret
  def scanfile(self,files,dirname=''):
      if self.level==None:
        raise ScannerError,'Forced error by const() scanner'
      for fn in files:
        debug.echo(6,self.name,": Scanning file: ",fn)
      return self.level,self.vir,self.ret

#####################################################################
### max_file_size class

class max_file_size(ascanner):
  '''
  Realscanner to test email's size.
  
  Usage: max_file_size(size,name='FileSizeOverrun')
  
  Where: size is a number of bytes, which if exceeded, virus is returned
         name is a string, which identifies the virus name.
            If this parameter is not present, 'FileSizeOverrun' is returned.

  Example: max_file_size(1024*1024*10)
           # all files at least 10MB will be reported as virus
  '''
  name='max_file_size()'
  def __init__(self,size,vname='FileSizeOverrun'):
      self.size=size
      self.vname=vname
  def scanstream(self,stream):
      if len(stream)<=self.size:
        return 0.0,'',[]
      else:
        return 1.0,self.vname,[self.filename+": "+self.vname]
  def scanfile(self,files,dir=''):
      for fn in files:
        if os.lstat(safe.fn(fn))[6]>self.size:
          return 1.0,self.vname,[fn+': '+self.vname]
      return 0.0,'',[]

#####################################################################
### exec_any class

class exec_any(ascanner):
  '''
  Realscanner to exec a defined command. [obsolete]
  
  If command returns exit status 0, no virus is returned.
  If exit code is any from statusX, virus is returned.
  Otherwise an error is raised.
  
  Usage: exec_any([command,arg1,...],[statusA1,...],[statusB1,...])
  
  Where: command is a string, which command to run
         argX are command line arguments, as the last argument the
           filename is added
         statusA1,... are integers, which defines "clean" exit codes
         statusB1,... are integers, which defines "virus found" exit codes
  
  This scanner is obsolete since 0.5.9. Use cmd() scanner.
  '''
  name='exec_any()'
  def __init__(self,arg,clean_status=[0],infected_status=[1]):
      self.arg=arg
      self.clean_status=clean_status
      self.infected_status=infected_status
  def scanfile(self,files,dir=''):
      for fn in files:
        pf=popen(self.arg+[fn])
        ret=pf.readlines()
        pf.wait()
        pf.close()
        if pf.exitstatus in self.clean_status:
          return 0.0,'',[]
        else:
          if pf.exitstatus in self.infected_status:
            return 1.0,'UnknownVirus',ret+['ExitCode: '+str(pf.exitstatus)]
          else:
            raise ScannerError,'UnkonwnExitStatus: '+str(pf.exitstatus)

class file_type(ascanner):
  '''
  Realscanner, which scans type of a file.
  
  A scanner to chcek content of a file for a special type (like zip, ...).
  
  Usage: file_type({'type':'vir',...})
  
  Where 'type' is a name of type returned by filetype.* function
        'vir' is a vir name returned, if file type is matched
  
  Example: # scan for executables (COM,EXE,PIF and similiar types)
           parsemail(file_type({'exe': 'MS Executable',
                                'elf': 'Linux Executable'}))
  '''
  name='file_type()'
  def __init__(self,types):
      self.types=types
  def scanstream(self,stream):
      fta=filetype.what(stream)
      for ft in fta:
        if ft in self.types:
          return 1.0,self.types[ft],[self.filename+": "+str(fta)]
      return 0.0,'',[]
  def scanfile(self,files,dirname=''):
      for fname in files:
        fta=filetype.file(fname)
        for ft in fta:
          if ft in self.types:
            return 1.0,self.types[ft],[fname+": "+str(fta)]
      return 0.0,'',[]

class file_magic(ascanner):
  '''
  File magic test (like "file -i command").
  
  This scanner can be used to test file content for a special type.
  You need a python module "magic", which is by default in file's
  source, but in most distributions it is not compiled in package.
  
  Usage: file_magic({'VirName':'regexp_pattern',..}, flags=0)
  
  Where: 'VirName' is a string, which identifies defined virus
         'regexp_pattern' is a regular expression pattern
         flags is an int, which defines regular expression flags,
           like re.I to ignore it's case.
  '''
  name='file_magic()'
  def __init__(self,magics,flags=re.I):
      import magic
      self.magic=magic.open(magic.MAGIC_MIME)
      self.magic.load()
      # compile regexps
      self.magics={}
      for vname,reg in magics.items():
        self.magics[vname]=re.compile(reg,flags)
  def __del__(self):
      self.magic.close()
  def scanstream(self,stream):
      for vname,reg in self.magics.items():
        if reg.search(self.magic.buffer(stream)):
          mail.addheader("X-Sagator-FileMagic",vname)
          return iret(1.0,vname,['file_magic: '+vname+' found in '+self.filename+'!\n'])
      return 0.0,'',[]
  def scanfile(self,files,dirname=''):
      for fname in files:
        for vname,reg in self.magics.items():
          if reg.search(self.magic.file(fname)):
            return iret(1.0,vname,[self.name+": "+vname+' found in '+self.filename+'!\n'])
      return 0.0,'',[]
