## Sagator configuration file.
## (c) 2003-2004 Jan ONDREJ (SAL) <ondrejj|a|salstar.sk>

## Lines beginning with double # (##) are comments. Lines beginning
## with single # are commented examples. In default configuration
## there is one antivir scanner and one spam scanner uncomented,
## other scanner are commented out.

## Debugging level, 0=errors only, 1=return status, init messages,
##   2=smtp server communication, 3=detailed smtp server communication,
##   4=tracebacks, 5=smtp client communication
##   Do not use debug level higher than 9!
DEBUG_LEVEL=3

## Language used by web access
##   You can define a locale language for which there are translations.
LANG=['en_US']

## Where is new root path. For example '/var/spool/vscan'
## Comment out this line, if you don't need to run sagator in chroot.
CHROOT='/var/spool/vscan'

## Logfile (use logfile in chroot to allow rotating)
LOGFILE=CHROOT+'/var/log/sagator/sagator.log'

## User and group, under which this program runs.
USER,GROUP='vscan','vscan'

## SMTP server host and port. You must define this smtp server
## in postfix without filtering.
SMTP_SERVER=('127.0.0.1',26)

## Scanners and services
from scanners import *
from srv import *

DROP='(worm|@mm|trojan|Sobig|Klez|Bugbear|Gibe|Swen|Mimail|Sober|Bagle|Mydoom|Novarg|Lirva)'

## Now you can define SCANNERS array. This array contains definitions
## for all scanners used in sagator and it's scripts.
## You can define other array names for different services.
## SCANNERS array for sgscan must have this name.
SCANNERS=[

  ## We are defining an logger scanner. This scanner will log some
  ## special data into logfile. See log() scanner documentation
  ## for more information. You can comment out this line, if you don't
  ## need extra information in your logs.
  log(1,log.SUMMARY_REPORT,
  
  ## Also you can use SQL logger. If you uncomment a scanner here,
  ## do not forget to uncomment it's parenthesis below!
  #log_sql(db.MySQLdb(host='127.0.0.1',port=3306,
  #                   dbname='sagator',dbuser='sagator',dbpasswd='your_pass'),
  #        log_sql.FORMAT,
  #log_sql(db.sqlite(dbname='/var/lib/sagator.sqlite'), log_sql.FORMAT,

    ## Next scanner defines a status line for statistics collector.
    ## This line stores "Virus" count in collector.
    ## You don't need it if you don't need MRTG statistics.
    status("Virus",
      ## If you need to send some virus reports to adminstrator,
      ## you can use following line. For of message template (MSG_TMPL)
      ## syntax read scanner documentation and/or source.
      ## You can comment out this, if you don't need reports.
      report(['root@localhost'],report.MSG_TMPL,
        ## Following scanner can quarantine all infected emails into
        ## files on server. This example quarantines files into
        ## a directory named /tmp/quarantine/... in sagator's chroot.
        ## In this directory there will be each subdirectory for each
        ## day (for example 200409).
        quarantine('/tmp/quarantine/%Y%m','',
          ## Following scanner defines, if you are need to reject, drop
          ## or deliver messages with viruses. By default viruses are
          ## rejected (and sent back to sender). Some viruses fakes
          ## it's sender and it is better do drop these emails.
          ## You can define virus names, which you want to drop.
          ## DROP variable is defined above.
          drop(DROP,
          
            ## Antivirus scanners follows here.
            
            ## Simple scanners
            ## Following scanner reports as virus all email larger than 10kB.
            #max_file_size(10*1024),
            ## Following scanner parses email for attachments and if
            ## one of them is executable, virus will be identified.
            #parsemail(file_type({'exe': 'Executable'})),
            ## Next scanner scans for viruses, if you can define a pattern,
            ## which is contained in each virus of this type.
            ## You can use it for it's own purposes to stop delivering
            ## of any king of emails.
            #string_scan(VIR_PATTERNS),
            ## This scanner is similiar to previous. It scans for regular
            ## expressions.
            #regexp_scan({'virname':['___PATTERN___']}),
            ## Exec any program
            ## You can use this scanner for unsuported antivirus,
            ## if you can define, which exit statuses are returned
            ## for viruses and for clean emails.
            #s2f(exec_any(['/bin/grep','-q','^TVqQ'],[1],[0])),

            ## ClamAV - clam antivirus
            ## Uncoment one or more following lines.
            #alternatives(
              ## Next scanner uses clamav's library directly in sagator.
              ## This scanner is the best scanner from all clamav scanners.
              ## It's performance and stability is very high.
              stream2mbox(libclam(limits={'maxratio':9999})),
              ## If you need to parse emails mime attachments, you
              ## can use parsemail() interscanner before calling clamav.
              ## Uncomment following line if you need this.
              ## Don't forget to comment out previous scanner, because
              ## it is useless to define two scanner for one antivirus.
              #parsemail(libclam()),
              ## Next scanner adds sagator's own decompression for clamav.
              ## It is only an example. You can use it for antivirs,
              ## which hasn't this feature implemented.
              #parsemail(stream2file(decompress(libclam()))),
              ## Next scanner calls clamav scan over clamav's daemon.
              ## This daemon is waiting on local port 3310/tcp.
              #clamd(['127.0.0.1',3310]),
              ## Next scanner calls clamav scan over clamav's daemon.
              ## This daemon is waiting on socket /var/run/clamav/clamd.sock.
              #clamd('/var/run/clamav/clamd.sock'),
              ## Following scanner is obsolete. It calls clamscan binary
              ## to scan for viruses. This scanner is very slow.
              #stream2mbox(clamscan(['/usr/bin/clamscan','--stdout','--infected','--disable-summary','-r','--mbox'])),
            #),

            ## AVG7 for linux
            ## This scanner can be used with AVG antivirus for linux.
            ## Uncomment next line, if you have it.
            #parsemail(s2f(avgd(chroot=CHROOT))),

            ## Bitdefender bdc
            ## This scanner can be used with bitdefender antivirus.
            ## Uncomment next line, if you have it.
            #s2f(bdc()),

            ## NOD32 (by ESET)
            ## There are three ways to use this antivirus.
            ## Following scanner uses nod32pac (preload library) as scaner.
            #nod2pac('/tmp/nod32/'),
            ## Next scanner uses nod32 version 2 as command line scanner.
            #stream2mbox(nod2()),
            ## Next scanner uses nod32lfs's dazuko support.
            #nod2dazuko('/tmp/dazuko/mb-','/var/log/nod32fac.log'),

            ## Sophie (sophos libsavi)
            ## Following scanner can be used with Sophie. Sophie
            ## is a daemon which uses libsavi library from Sophos antivirus.
            #parsemail(s2f(decompress(sophie('/tmp/sophie',CHROOT)))),
            
            ## Kaspersky antivirus
            ## You can use following scanner for Kaspersky antivurus
            ## command line scanner.
            #s2f(kav()),

            ## Symantec antivirus scan engine.
            ## You can use following scanner for Symantec antivurus
            ## scan engine. Do not forget to configure ICAP protocol
            ## on port 1344.
            #savse('127.0.0.1',1344),
          )
        )
      )
    ),
    ## Now we are defining status for "Spam",
    status("Spam",
      ## quarantine for spams,
      quarantine('/tmp/quarantine/%Y%m','',
        # and dropping of all spams.
        drop('.', # drop all spams

          ## Antispam scanners follows here.
          ## SpamAssassin
          ## This scanner using default configuration for spamd
          ## (spamassassin daemon) on local port 783/tcp.
          ## It is using spamassassin's default configuration.
          spamassassind(['127.0.0.1',783]),

          ## Bogofilter
          #bogofilter(['/usr/bin/bogofilter','-v']),

          ## QuickSpamFilter
          #qsf(['/usr/bin/qsf','-r']),
          
          ## Anomy Sanitizer
          #filter(['/usr/local/bin/sanitizer.pl'])
        )
      )
    )
  #)
  )
]

## LMTP scanner dictionary example:
## This definition is very simple. Use SCANNERS konfiguration for more
## examples and read sagator's documentation.
#LMTP_SCANNERS={
#  'antivir_only:
#      log(1,log.SUMMARY_REPORT,
#        quarantine('/tmp/quarantine/%Y%m','',
#          drop(DROP,
#            stream2mbox(libclam(limits={'maxratio':9999}))
#          )
#        )
#      )
#  'DEFAULT': # 'DEFAULT' string is hardcoded
#      SCANNERS[0], # define this as first scanner from SCANNERS
#}

## In this section you need to define services, which will be started
## by SAGATOR. You need at least one service to start. An SMTP gateway
## or a command can communicate with SAGATOR over this/these services.
SRV=[
  ## External daemons used by SAGATOR
  ## Uncomment following line, if you want to use clamd in chroot.
  #chroot_execvp('/usr/sbin/clamd', ['-c', '/etc/clamav.conf']),
  ## Uncomment following line, if you want to use AVG daemon in chroot.
  #chroot_execvpe('/opt/grisoft/avg7/bin/avgscan', ['-d'],
  #               {'LANG':'C'}, pgrp_file='/var/run/avgd.pgrp'),
  ## Uncomment following line, if you want to use KAV daemon in chroot.
  #chroot_execvp('/opt/kav/5.5/kav4mailservers/bin/aveserver'),

  ## Resource limits (like ulimit)
  ## You can define resource limits for sagator processes.
  ## In this example address space is limited to 50 MB.
  #rlimit(AS=70*MB),

  ## Statistics collector
  ## This service can be used to collect statistics data and an program
  ## (like MRTG) can use these data to show nice graphs. By default
  ## leave this service running, because there is a script in sagator,
  ## which using this service.
  collector('127.0.0.1',28),

  ## SMTP daemon (for postfix, ...)
  ## This service can be used by postfix or any other SMTP daemon.
  ## You need to configure your SMTPd to send all viruses over
  ## this SMTPd. It sends clean emails back to SMTPd defined above
  ## (by SMTP_SERVER variable).
  smtpd(SCANNERS,'127.0.0.1',27)

  ## LMTP daemon (for postfix, ...)
  ## This service can be used to scan each email recipient with different
  ## scanner. Configure postfix to use lmtp protocol (lmtp:IP:port).
  #lmtpd(LMTP_SCANNERS,
  #  e2i_regexp([['^an_regexp$','SCANNER_NAME']]),
  #  '0.0.0.0',27
  #),

  ## Milter daemon
  ## This service can be used by sendmail's milter. Leave it commented,
  ## if you don't use sendmail SMTP.
  #milter(SCANNERS,"sagator","inet:3333@127.0.0.1")

  ## Standard input filter
  ## Over this service you can use sagator as STDIN -> STDOUT filter.
  ## Configure avfilter() and run sagator:
  ##   sagator --nodaemon < email
  ## and you will obtain modified email on standard output.
  #avfilter(SCANNERS)

  ## HTTP proxy filter
  ## This service can be used to scan HTTP connection for viruses.
  ## Please read proxy() service documentation for client configuration.
  ## WARNING: This service is in beta stage. USE WITH CAUTION!
  #http_proxy(SCANNERS,'127.0.0.1',3129)
  
  ## Reporter virtual service
  #reporter()
  
  ## Web access virtual service
  #webq_apache(
  #  db=db.MySQLdb(host='localhost', port=3306, dbname='sagator',
  #                dbuser='sagator', dbpasswd='SAGATORS_DB_PASSWORD'),
  #  scanner=s2f(libclam()),
  #  admin=['root@localhost']
  #)
]
