'''
daemon.py - daemon starter service for sagator

(c) 2005 Jan ONDREJ (SAL) <ondrejj(at)salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

'''

from aglib import *

__all__=['chroot_execvp','chroot_execvpe']

class chroot_execvp(service):
  '''
  Execute an external command (or start an daemon).
  
  This service can be used to start external daemons used by sagator.
  
  Usage: chroot_execvp('daemon_name',args=[],pid_file='')
  
  Where: daemon_name is a string, which defines command name
         args are strings, which defines command line arguments
         pid_file is an string to filename with PID of started daemon
         pgrp_file is an string to a process group filename
  
  Example: chroot_execvp('/usr/sbin/clamd','-c','/etc/clamav.conf')
  
  New in version 0.7.0.
  '''
  name='chroot_execvp()'
  def __init__(self,command,args=[],pid_file='',pgrp_file=''):
      self.CMD=[command]+list(args)
      self.ENV={}
      self.PID_FILE=pid_file
      self.PGRP_FILE=pgrp_file
  def stop(self):
      if self.PID_FILE:
        try:
          pid=int(open(globals.CHROOT+self.PID_FILE).read())
          debug.echo(1,'%s: Killing %s pid=%d' % (self.name,self.PID_FILE,pid))
          os.kill(pid,signal.SIGTERM)
        except IOError:
          debug.echo(1,"%s: Can't open file %s" % (self.name,self.PID_FILE))
        except OSError,(ec,es):
          debug.echo(0,"%s: killpg: %s" % (self.name,es))
      if self.PGRP_FILE:
        try:
          pgrp=int(open(globals.CHROOT+self.PGRP_FILE).read())
          debug.echo(1,'%s: Killing %s pgrp=%d' % (self.name,self.PGRP_FILE,pgrp))
          os.killpg(pgrp,signal.SIGTERM)
        except AttributeError:
          # python 2.2 or later? Try to use kill command.
          os.system('/bin/kill -TERM -- -%d' % pgrp)
        except IOError:
          debug.echo(1,"%s: Can't open file %s" % (self.name,self.PGRP_FILE))
        except OSError,(ec,es):
          debug.echo(0,"%s: killpg: %s" % (self.name,es))
  def start(self):
      self.pid=os.fork()
      if self.pid==0:
        debug.echo(1,'chroot_execvp(): %s' % self.CMD)
        try:
          os.chroot(globals.CHROOT)
          if self.ENV:
            os.execvpe(self.CMD[0],self.CMD,self.ENV)
          else:
            os.execvp(self.CMD[0],self.CMD)
        except OSError,(ec,es):
          debug.echo(0,"%s: WARNING: Can't start process in chroot: %s" \
                     % (self.name,es))
      return [self.pid]

class chroot_execvpe(chroot_execvp):
  '''
  Execute an external command (or start an daemon) and update variables.
  
  This service can be used to start external daemons used by sagator.
  You also can set environment varibles.
  
  Usage: chroot_execvpe('daemon_name',args=[],env={})
  
  Where: daemon_name is a string, which defines command name
         args are strings, which defines command line arguments
         env is an dictionary of environment variables
         pid_file is an string to filename with PID of started daemon
         pgrp_file is an string to a process group filename
  
  Example: chroot_execvp('/usr/sbin/clamd','-c','/etc/clamav.conf',
                         {'LANG': 'C'})
  
  New in version 0.7.0.
  '''
  name='chroot_execvpe()'
  def __init__(self,command,args=[],env={},pid_file='',pgrp_file=''):
      self.CMD=[command]+list(args)
      self.ENV=env
      self.PID_FILE=pid_file
      self.PGRP_FILE=pgrp_file
