'''
smtpd.py - SMTP daemon service for sagator.

(c) 2003-2004 Jan ONDREJ (SAL) <ondrejj(at)salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

'''

from aglib import *

__all__=['smtpd']

class smtps(smtp):
  '''
  SMTP server class
  '''
  reg_xforward=re.compile("^XFORWARD ",re.IGNORECASE)
  reg_rset=re.compile("^RSET",re.IGNORECASE)
  def __init__(self,scanners,conn,addr):
    self.SCANNERS=scanners
    self.f=conn.makefile('rw',self.bufsize)
    self.write=conn.sendall
    self.readline=self.f.readline
    self.RECV=RECV_SMTP
    self.t0=time.time()
    self.srv=smtpc()
    self.write(self.srv.readline())
    mail.__init__()
    mail.addr='NOOP CONNECT FROM: '+addr[0]+':'+str(addr[1])\
      +', '+mail.getnamebyaddr(addr[0])+'\r\n'
    while True:
      if self.recv(): break
  def send_ok(self,r="Ok"):
    self.write("250 "+r+"\r\n")
  def quit_connect(self):
    debug.echo(5,"SMTPS: Closing connect to server")
    self.RECV=RECV_QUIT
    self.srv.close()
    return self.recv()
  def check_reply(self,line):
    '''Check for SMTP reply. Return True, if 2XX or 3XX returned.'''
    self.srv.conn.sendall(line)
    # send reply
    while 1:
      l=self.srv.readline()
      if not l:
        debug.echo(3,"SMTPS: Connection closed after:",line)
        return False
      debug.echo(7,'SMTPC: '+l)
      self.write(l)
      if self.reg_smtp_reply.search(l):
        break
    if self.reg_smtp_reply_ok.search(l):
      return True
    debug.echo(4,"SMTPS: "+l.rstrip()+' --- '+line)
    return False
  def recv(self):
    try:
      line=self.readline()
    except IOError,(ec,es):
      debug.echo(0, "smtps(): ",es)
      return 1
    if not line:
      debug.echo(9, "smtps(): Empty line")
      return 1
    if self.RECV: # receiving data or header
      if self.RECV==RECV_QUIT: # waiting for quit?
        if self.reg_quit.search(line):
          self.write("221 Bye\r\n")
          self.f.close()
          return 1
        else:
          self.write("502 SMTP communication stopped by Sagator\r\n")
          return 0
      elif (line=='.\r\n') | (line=='.\n'):
        # end of DATA part
        mail.close()
        debug.echo(3,"SMTPS: BODY DONE, size: ",len(mail.data)," B")
        self.RECV=RECV_SMTP
        v,virname=checkvir(self.SCANNERS)
        self.tw=time.time()-self.t0
        if v!=S_TEMPFAIL:
          debug.echo(2,"STATS: "+str(self.tw)+
            " seconds, ",len(mail.data)," bytes, status: ",virname)
        if (v==S_OK) | (v==S_FORCE_SEND):
          debug.echo(5,"SMTPS: Sending data")
          self.srv.conn.sendall(mail.xheader)
          self.srv.conn.sendall(mail.data)
          debug.echo(1,"SMTPS: OK: 250 Ok")
          self.srv.conn.sendall(line) # send .
          self.write(self.srv.readline())
          stats.update(len(mail.data),self.tw)
          mail.__init__() # reinit
        elif v==S_REJECT:
          debug.echo(1,"SMTPS: REJECT: ","550 Content rejected - ",virname)
          self.write("550 Content rejected - "+str(virname)+"\r\n")
          stats.update(len(mail.data),self.tw,1)
          return self.quit_connect()
        elif v==S_DROP:
          debug.echo(1,"SMTPS: DROP: 250 mail dropped - ",virname)
          self.write("250 mail dropped - "+str(virname)+"\r\n")
          stats.update(len(mail.data),self.tw,1)
          return self.quit_connect()
        else: # S_TEMPFAIL
          debug.echo(1,"SMTPS: TEMPFAIL: 451 ",virname)
          self.write("451 "+str(virname)+"\r\n")
          stats.update(0,0,0,1) # update fail statistics
          return self.quit_connect()
      else:
        mail.df.write(line)
    else:
      mail.comm=mail.comm+line
      rcptto=self.reg_rcptto.search(line)
      mailfrom=self.reg_mailfrom.search(line)
      if rcptto:
        if self.check_reply(line):
          debug.echo(2,'SMTPS: '+line)
          try:
            mail.recip.append(rfc822.parseaddr(rcptto.group(1))[1])
          except:
            pass
        return 0
      elif mailfrom:
        if self.check_reply(line):
          debug.echo(2,'SMTPS: '+line)
          try:
            mail.sender=rfc822.parseaddr(mailfrom.group(1))[1]
          except:
            mail.sender=mailfrom.group(1)
      elif self.reg_rset.search(line):
        if self.check_reply(line):
          debug.echo(2,'SMTPS: '+line.rstrip(),", Resetting mail class")
          mail.__init__() # reinit
      elif self.reg_data.search(line):
        if self.check_reply(line):
          debug.echo(5,'SMTPS: '+line)
          self.RECV=RECV_BODY
      elif self.reg_quit.search(line):
        debug.echo(5,'SMTPS: '+line)
        self.check_reply(line) # 221 Bye
        return 1
      elif self.reg_xforward.search(line):
        debug.echo(2,'SMTPS: ',line)
        self.check_reply(line)
      else:
        debug.echo(6,'SMTPS: '+line)
        self.check_reply(line)
    return 0

class smtpd(service):
  '''
  SMTP daemon service.
  
  This service can be used to start sagator as separate filtering SMTP
  daemon. Is is useful for postfix and any other SMTP daemon, which
  can use these filters.
  
  Usage: smtpd(scanners, host, port, prefork=2)
  
  Where: scanners is an array of scanners (see README.scanners for more info)
         host is a an ip address to bind
         port is a port to bind
         prefork is a number, which defines preforked process count.
           Set this parameter to actual processor count + 1
           or leave it's default (2).
  
  Example: smtpd(SCANNERS, '127.0.0.1', 27)
  '''
  name='smtpd()'
  def accept(self,connects=0):
      # accept
      conn,addr = self.s.accept()
      socket_settimeout(conn,120)
      # reinit scanners
      for scnr in self.SCANNERS:
        scnr.reinit()
      globals.reset()
      # generate ID
      self.time2=time.strftime("%Y%m%d-%H%M%S",time.localtime(time.time()))
      if self.time2!=self.time1:
        self.time1,self.timec=self.time2,1
      globals.id="%s-%04d-%05d-%s" % (self.time2,self.timec,os.getpid(),randomchars(6))
      debug.echo(1,"Connection from: "+addr[0]+" at "+\
        time.strftime("%c",time.localtime()))
      debug.echo(1,"Process id: %s" % globals.id)
      smtps(self.SCANNERS,conn,addr)
      debug.echo(1,"Closing connection.")
      conn.shutdown(2)
      conn.close()
